// 创建职工表结构
public void createEmpTable() throws IOException {
    HBaseAdmin admin = new HBaseAdmin(conf);
    if (admin.tableExists("emp1520")) {
        System.out.println("Table already exists");
        return;
    }
    HTableDescriptor tableDesc = new HTableDescriptor(TableName.valueOf("emp1520"));
    tableDesc.addFamily(new HColumnDescriptor("empnum")); // 存储员工ID
    tableDesc.addFamily(new HColumnDescriptor("info")); // 存储基本信息
    tableDesc.addFamily(new HColumnDescriptor("salary")); // 存储薪资
    tableDesc.addFamily(new HColumnDescriptor("performance")); // 存储绩效信息
    tableDesc.addFamily(new HColumnDescriptor("training")); // 存储培训课程
    admin.createTable(tableDesc);
    admin.close();
}

// 插入数据实现
public void loadData() throws IOException {
    Table table = connection.getTable(TableName.valueOf("emp1520"));
    try (BufferedReader br = new BufferedReader(new FileReader("emp.txt"))) {
        String line;
        while ((line = br.readLine()) != null) {
            String[] fields = line.split(",");
            String empno = fields[0];
            String rowKey = generateRowKey(empno); // 生成MD5前缀+empno的RowKey
            Put put = new Put(Bytes.toBytes(rowKey));
            // empnum列族
            put.addColumn(Bytes.toBytes("empnum"), Bytes.toBytes("empno"), Bytes.toBytes(empno));
            // info列族
            put.addColumn(Bytes.toBytes("info"), Bytes.toBytes("ename"), Bytes.toBytes(fields[1]));
            put.addColumn(Bytes.toBytes("info"), Bytes.toBytes("job"), Bytes.toBytes(fields[2]));
            put.addColumn(Bytes.toBytes("info"), Bytes.toBytes("mgr"), Bytes.toBytes(fields[3]));
            put.addColumn(Bytes.toBytes("info"), Bytes.toBytes("hiredate"), Bytes.toBytes(fields[4]));
            put.addColumn(Bytes.toBytes("info"), Bytes.toBytes("credit"), Bytes.toBytes(fields[5]));
            // salary列族
            put.addColumn(Bytes.toBytes("salary"), Bytes.toBytes("sal"), Bytes.toBytes(fields[6]));
            // performance列族
            put.addColumn(Bytes.toBytes("performance"), Bytes.toBytes("work_hours"), Bytes.toBytes(fields[7]));
            put.addColumn(Bytes.toBytes("performance"), Bytes.toBytes("performance_rating"), Bytes.toBytes(fields[8]));
            put.addColumn(Bytes.toBytes("performance"), Bytes.toBytes("promotion_date"), Bytes.toBytes(fields[9]));
            // training列族
            put.addColumn(Bytes.toBytes("training"), Bytes.toBytes("training_courses"), Bytes.toBytes(fields[10]));
            table.put(put);
        }
    }
}

// RowKey生成方法
private String generateRowKey(String empno) {
    String prefix = empno.substring(0,3);
    String md5 = md5Hash(prefix);
    return md5.substring(0,4) + empno; // 前4位MD5+原empno
}

// MD5计算方法
private static String md5Hash(String input) {
    try {
        MessageDigest md = MessageDigest.getInstance("MD5");
        byte[] messageDigest = md.digest(input.getBytes());
        BigInteger number = new BigInteger(1, messageDigest);
        return number.toString(16);
    } catch (NoSuchAlgorithmException e) {
        throw new RuntimeException(e);
    }
}

// 查询ID>7500的员工
public void queryEmpNoOver7500() throws IOException {
    Scan scan = new Scan();
    Filter filter = new SingleColumnValueFilter(
            Bytes.toBytes("empnum"),
            Bytes.toBytes("empno"),
            CompareFilter.CompareOp.GREATER_OR_EQUAL,
            new SubstringComparator("7500")
    );
    scan.setFilter(filter);
    ResultScanner scanner = table.getScanner(scan);
    for (Result result : scanner) {
        // 输出结果处理
    }
}

// 绩效>4且入职早于2022年
public void queryPerformanceAndHiredate() throws IOException {
    FilterList list = new FilterList(FilterList.Operator.MUST_PASS_ALL);
    list.addFilter(new SingleColumnValueFilter(
            Bytes.toBytes("performance"),
            Bytes.toBytes("performance_rating"),
            CompareFilter.CompareOp.GREATER_OR_EQUAL,
            new SubstringComparator("5")
    ));
    list.addFilter(new SingleColumnValueFilter(
            Bytes.toBytes("info"),
            Bytes.toBytes("hiredate"),
            CompareFilter.CompareOp.LESS,
            new SubstringComparator("2022-01-01")
    ));
    Scan scan = new Scan();
    scan.setFilter(list);
    ResultScanner scanner = table.getScanner(scan);
    // 结果处理逻辑
}

// 最近晋升员工查询
public void findLatestPromotion() throws IOException {
    Scan scan = new Scan();
    ResultScanner scanner = table.getScanner(scan);
    List<PromotionRecord> records = new ArrayList<>();
    for (Result result : scanner) {
        String promotionDateStr = Bytes.toString(
                result.getValue(Bytes.toBytes("performance"), Bytes.toBytes("promotion_date"))
        );
        if (promotionDateStr != null) {
            records.add(new PromotionRecord(
                    Bytes.toString(result.getRow()),
                    LocalDate.parse(promotionDateStr)
            ));
        }
    }
    records.sort(Comparator.comparing(PromotionRecord::getDate).reversed());
    // 输出最新记录
}

// 培训课程统计
public void countTrainingCourses() throws IOException {
    Map<String, Integer> courseMap = new HashMap<>();
    Scan scan = new Scan();
    ResultScanner scanner = table.getScanner(scan);
    for (Result result : scanner) {
        String coursesStr = Bytes.toString(
                result.getValue(Bytes.toBytes("training"), Bytes.toBytes("training_courses"))
        );
        if (coursesStr != null) {
            for (String course : coursesStr.split(",")) {
                courseMap.put(course, courseMap.getOrDefault(course, 0)+1);
            }
        }
    }
    // 输出统计结果
}
